@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2005 - 2011 Vivien Malerba <malerba@gnome-db.org>
- * Copyright (C) 2005 Ãlvaro PeÅa <alvaropg@telefonica.net>
- * Copyright (C) 2008 PrzemysÅaw Grzegorczyk <pgrzegorczyk@gmail.com>
+ * Copyright (C) 2005 Álvaro Peńa <alvaropg@telefonica.net>
+ * Copyright (C) 2008 Przemysław Grzegorczyk <pgrzegorczyk@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -30,15 +30,7 @@
 G_BEGIN_DECLS
 
 #define GDA_TYPE_COLUMN            (gda_column_get_type())
-#define GDA_COLUMN(obj)            (G_TYPE_CHECK_INSTANCE_CAST (obj, GDA_TYPE_COLUMN, GdaColumn))
-#define GDA_COLUMN_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST (klass, GDA_TYPE_COLUMN, GdaColumnClass))
-#define GDA_IS_COLUMN(obj)         (G_TYPE_CHECK_INSTANCE_TYPE (obj, GDA_TYPE_COLUMN))
-#define GDA_IS_COLUMN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GDA_TYPE_COLUMN))
-
-struct _GdaColumn {
-	GObject           object;
-	GdaColumnPrivate *priv;
-};
+G_DECLARE_DERIVABLE_TYPE (GdaColumn, gda_column, GDA, COLUMN, GObject)
 
 struct _GdaColumnClass {
 	GObjectClass          parent_class;
@@ -65,12 +57,11 @@ struct _GdaColumnClass {
  * The #GdaColumn object represents a #GdaDataModel's column and handle all its properties.
  */
 
-GType           gda_column_get_type           (void) G_GNUC_CONST;
 GdaColumn      *gda_column_new                (void);
 GdaColumn      *gda_column_copy               (GdaColumn *column);
 
 const gchar    *gda_column_get_description          (GdaColumn *column);
-void            gda_column_set_description          (GdaColumn *column, const gchar *title);
+void            gda_column_set_description          (GdaColumn *column, const gchar *descr);
 
 const gchar    *gda_column_get_name           (GdaColumn *column);
 void            gda_column_set_name           (GdaColumn *column, const gchar *name);
@@ -93,15 +84,12 @@ void            gda_column_set_position       (GdaColumn *column, gint position)
 const GValue   *gda_column_get_default_value  (GdaColumn *column);
 void            gda_column_set_default_value  (GdaColumn *column, const GValue *default_value);
 
-const GValue   *gda_column_get_attribute      (GdaColumn *column, const gchar *attribute);
-void            gda_column_set_attribute      (GdaColumn *column, const gchar *attribute, const GValue *value,
-					       GDestroyNotify destroy);
 
 /**
  * gda_column_set_attribute_static:
  * @holder: a #GdaHolder
  * @attribute: attribute's name
- * @value: (allow-none): the value to set the attribute to, or %NULL
+ * @value: (nullable): the value to set the attribute to, or %NULL
  *
  * This function is similar to gda_column_set_attribute() but for static strings
  */
