@@ -1,12 +1,12 @@
 /*
  * Copyright (C) 2000 Reinhard Müller <reinhard@src.gnome.org>
  * Copyright (C) 2000 - 2005 Rodrigo Moya <rodrigo@gnome-db.org>
- * Copyright (C) 2001 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2001 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2002 Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
  * Copyright (C) 2003 Laurent Sansonetti <laurent@datarescue.be>
  * Copyright (C) 2003 - 2007 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2005 Andrew Hill <andru@src.gnome.org>
- * Copyright (C) 2013 Daniel Espinosa <despinosa@src.gnome.org>
+ * Copyright (C) 2013, 2018 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,22 +32,14 @@
 
 G_BEGIN_DECLS
 
-#define GDA_TYPE_CONFIG            (gda_config_get_type())
-#define GDA_CONFIG(obj)            (G_TYPE_CHECK_INSTANCE_CAST (obj, GDA_TYPE_CONFIG, GdaConfig))
-#define GDA_CONFIG_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST (klass, GDA_TYPE_CONFIG, GdaConfigClass))
-#define GDA_IS_CONFIG(obj)         (G_TYPE_CHECK_INSTANCE_TYPE(obj, GDA_TYPE_CONFIG))
-#define GDA_IS_CONFIG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), GDA_TYPE_CONFIG))
-
-typedef struct _GdaConfigPrivate GdaConfigPrivate;
 typedef struct _GdaDsnInfo GdaDsnInfo;
-typedef struct _GdaProviderInfo GdaProviderInfo;
 
 /* error reporting */
 extern GQuark gda_config_error_quark (void);
 #define GDA_CONFIG_ERROR gda_config_error_quark ()
 
 typedef enum {
-        GDA_CONFIG_DSN_NOT_FOUND_ERROR,
+	GDA_CONFIG_DSN_NOT_FOUND_ERROR,
 	GDA_CONFIG_PERMISSION_ERROR,
 	GDA_CONFIG_PROVIDER_NOT_FOUND_ERROR,
 	GDA_CONFIG_PROVIDER_CREATION_ERROR
@@ -84,8 +76,9 @@ struct _GdaDsnInfo {
 
 GType            gda_dsn_info_get_type  (void) G_GNUC_CONST;
 GdaDsnInfo*      gda_dsn_info_new       (void);
-GdaDsnInfo*      gda_dsn_info_copy      (GdaDsnInfo *source);
+GdaDsnInfo*      gda_dsn_info_copy      (const GdaDsnInfo *source);
 void             gda_dsn_info_free      (GdaDsnInfo *dsn);
+gboolean         gda_dsn_info_equal     (const GdaDsnInfo *dsn1, const GdaDsnInfo *dsn2);
 
 /**
  * GdaProviderInfo:
@@ -97,12 +90,13 @@ void             gda_dsn_info_free      (GdaDsnInfo *dsn);
  *
  * This structure holds the information associated to a database provider as discovered by Libgda.
  */
-struct _GdaProviderInfo {
+typedef struct {
         gchar             *id;
         gchar             *location;
         gchar             *description;
         GdaSet            *dsn_params;  /* Specs to create a DSN */
 	GdaSet            *auth_params; /* Specs to authenticate a client */
+	gchar             *icon_id; /* use gdaui_get_icon_for_db_engine(icon_id) to get the actual GdkPixbuf */
 
 	/*< private >*/
 	/* Padding for future expansion */
@@ -110,29 +104,12 @@ struct _GdaProviderInfo {
 	gpointer _gda_reserved2;
 	gpointer _gda_reserved3;
 	gpointer _gda_reserved4;
-};
+} GdaProviderInfo;
 
-struct _GdaConfig {
-	GObject           object;
-	GdaConfigPrivate *priv;
-};
+#define GDA_TYPE_PROVIDER_INFO (gda_provider_info_get_type ())
 
-struct _GdaConfigClass {
-	GObjectClass      object_class;
+GType            gda_provider_info_get_type  (void) G_GNUC_CONST;
 
-	/* signals */
-	void   (*dsn_added)                 (GdaConfig *conf, GdaDsnInfo *new_dsn);
-	void   (*dsn_to_be_removed)         (GdaConfig *conf, GdaDsnInfo *old_dsn);
-	void   (*dsn_removed)               (GdaConfig *conf, GdaDsnInfo *old_dsn);
-	void   (*dsn_changed)               (GdaConfig *conf, GdaDsnInfo *dsn);
-
-	/*< private >*/
-	/* Padding for future expansion */
-	void (*_gda_reserved1) (void);
-	void (*_gda_reserved2) (void);
-	void (*_gda_reserved3) (void);
-	void (*_gda_reserved4) (void);
-};
 
 /**
  * SECTION:gda-config
@@ -177,8 +154,27 @@ struct _GdaConfigClass {
  * gtk_init() calls setlocale().
  */
 
+G_DECLARE_DERIVABLE_TYPE (GdaConfig, gda_config, GDA, CONFIG, GObject)
+
+#define GDA_TYPE_CONFIG (gda_config_get_type ())
+
+struct _GdaConfigClass {
+	GObjectClass      parent_class;
+
+	/* signals */
+	void   (*dsn_added)                 (GdaConfig *conf, GdaDsnInfo *new_dsn);
+	void   (*dsn_to_be_removed)         (GdaConfig *conf, GdaDsnInfo *old_dsn);
+	void   (*dsn_removed)               (GdaConfig *conf, GdaDsnInfo *old_dsn);
+	void   (*dsn_changed)               (GdaConfig *conf, GdaDsnInfo *dsn);
+
+	/*< private >*/
+	/* Padding for future expansion */
+	void (*_gda_reserved1) (void);
+	void (*_gda_reserved2) (void);
+	void (*_gda_reserved3) (void);
+	void (*_gda_reserved4) (void);
+};
 
-GType              gda_config_get_type                 (void) G_GNUC_CONST;
 GdaConfig*         gda_config_get                      (void);
 
 GdaDsnInfo        *gda_config_get_dsn_info             (const gchar *dsn_name);
