@@ -1,16 +1,18 @@
 /*
- * Copyright (C) 2000 Reinhard MÃ¼ller <reinhard@src.gnome.org>
+ * Copyright (C) 2000 Reinhard Müller <reinhard@src.gnome.org>
  * Copyright (C) 2000 - 2004 Rodrigo Moya <rodrigo@gnome-db.org>
- * Copyright (C) 2001 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2001 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2002 Cleber Rodrigues <cleberrrjr@bol.com.br>
  * Copyright (C) 2002 - 2003 Gonzalo Paniagua Javier <gonzalo@ximian.com>
  * Copyright (C) 2003 Filip Van Raemdonck <mechanix@debian.org>
  * Copyright (C) 2004 - 2005 Alan Knowles <alank@src.gnome.org>
- * Copyright (C) 2004 JosÃ© MarÃ­a Casanova Crespo <jmcasanova@igalia.com>
+ * Copyright (C) 2004 José María Casanova Crespo <jmcasanova@igalia.com>
  * Copyright (C) 2005 - 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2006 - 2008 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2007 Leonardo Boshell <lb@kmc.com.co>
  * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2015 Gergely Polonkai <gergely@polonkai.eu>
+ * Copyright (C) 2018 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -38,14 +40,15 @@
 #include <libgda/gda-statement.h>
 #include <libgda/gda-meta-store.h>
 #include <libgda/gda-server-operation.h>
+#include <libgda/gda-batch.h>
+#include <libgda/gda-db-catalog.h>
+#include <libgda/gda-config.h>
 
 G_BEGIN_DECLS
 
 #define GDA_TYPE_CONNECTION            (gda_connection_get_type())
-#define GDA_CONNECTION(obj)            (G_TYPE_CHECK_INSTANCE_CAST (obj, GDA_TYPE_CONNECTION, GdaConnection))
-#define GDA_CONNECTION_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST (klass, GDA_TYPE_CONNECTION, GdaConnectionClass))
-#define GDA_IS_CONNECTION(obj)         (G_TYPE_CHECK_INSTANCE_TYPE(obj, GDA_TYPE_CONNECTION))
-#define GDA_IS_CONNECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), GDA_TYPE_CONNECTION))
+
+G_DECLARE_DERIVABLE_TYPE (GdaConnection, gda_connection, GDA,CONNECTION, GObject)
 
 /* error reporting */
 extern GQuark gda_connection_error_quark (void);
@@ -58,15 +61,16 @@ typedef enum {
 	GDA_CONNECTION_NO_CNC_SPEC_ERROR,
 	GDA_CONNECTION_NO_PROVIDER_SPEC_ERROR,
 	GDA_CONNECTION_OPEN_ERROR,
+	GDA_CONNECTION_ALREADY_OPENED_ERROR,
 	GDA_CONNECTION_STATEMENT_TYPE_ERROR,
-	GDA_CONNECTION_CANT_LOCK_ERROR,
+  GDA_CONNECTION_CANT_LOCK_ERROR,
 	GDA_CONNECTION_TASK_NOT_FOUND_ERROR,
-	GDA_CONNECTION_UNSUPPORTED_THREADS_ERROR,
 	GDA_CONNECTION_CLOSED_ERROR,
 	GDA_CONNECTION_META_DATA_CONTEXT_ERROR,
-	GDA_CONNECTION_UNSUPPORTED_ASYNC_EXEC_ERROR
+	GDA_CONNECTION_NO_MAIN_CONTEXT_ERROR
 } GdaConnectionError;
 
+
 #define GDA_CONNECTION_NONEXIST_DSN_ERROR GDA_CONNECTION_DSN_NOT_FOUND_ERROR
 
 /**
@@ -75,57 +79,62 @@ typedef enum {
  * @title: GdaConnection
  * @stability: Stable
  *
- * Each connection to a database is represented by a #GdaConnection object. A connection is created (and opened)
- * using gda_connection_open_from_dsn() if a data source has been defined, or gda_connection_open_from_string()
+ * Each connection to a database is represented by a #GdaConnection object. A connection is created
+ * using gda_connection_new_from_dsn() if a data source has been defined, or gda_connection_new_from_string()
  * otherwise. It is not recommended to create a #GdaConnection object using g_object_new() as the results are
  * unpredictable (some parts won't correctly be initialized).
  *
- * Use the connection object to execute statements, use transactions, get meta data information, ...
+ * Once the connection has been created, it can be opened using gda_connection_open() or gda_connection_open_async().
+ * By default these functions will block, unless you speficy a #GMainContext from which events will be processed
+ * while opening the connection using gda_connection_set_main_context().
  *
- * If supported by the database provider being used, statements can be executed asynchronously instead of
- * blocking the execution thread untill the execution of a statement is finished. Each database provider
- * is free to implement this feature as it wishes (using the API or using threads). The steps involved to
- * execute a statement are then:
- * <itemizedlist>
- *   <listitem><para>Request the statement execution using
- *	<link linkend="gda-connection-async-statement-execute">gda_connection_async_statement_execute() which returns an
- *	  execution ID to be used to identify a specific request</link></para></listitem>
- *   <listitem><para>Do some useful things (that is why async. statements' excution are for)</para></listitem>
- *   <listitem><para>Use one or more times 
- *	<link linkend="gda-connection-async-fetch-result">gda_connection_async_fetch_result()</link> to see
- *	if the execution is finished, using the request ID</para></listitem>
- *   <listitem><para>Use <link linkend="gda-connection-async-cancel">gda_connection_async_cancel()</link> to cancel
- * the execution of a statement</para></listitem>
- * </itemizedlist>
+ * The same gda_connection_set_main_context() allows the program to still handle events while a
+ * potentially blocking operation (such as executing a statement) is being carried out (this is a change compared to the
+ * #GdaConnection object in Libgda versions before 6.x, where asynchronous execution was featured
+ * using a set of specific functions like gda_connection_async_statement_execute()).
  *
- * The #GdaConnection object implements its own locking mechanism so it is thread-safe. However ad some database
- * providers rely on an API which does not support threads or supports it only partially, the connections
- * opened using those providers will only be accessible from the thread which created them (any other thread will
- * be blocked trying to access the connection, use the
- * <link linkend="gda-lockable-try-lock">gda_lockable_try_lock()</link> method to check it the connection
- * is usable from a thread).
- *
- * If a connection really needs to be accessed by several threads at once, then it is possible to pass the
- * #GDA_CONNECTION_OPTIONS_THREAD_SAFE flag when opening it. This flag requests that the real connection
- * be created and really accessed in a <emphasis>private</emphasis> sub thread.
+ * The #GdaConnection object implements its own locking mechanism so it is thread-safe. If a #GMainContext has been
+ * specified using gda_connection_set_main_context(), then the events will continue to be processed while the
+ * lock on the connection is being acquired.
  */
 
-struct _GdaConnection {
-	GObject               object;
-	GdaConnectionPrivate *priv;
-};
+/**
+ * GdaConnectionStatus:
+ * @GDA_CONNECTION_STATUS_CLOSED: the connection is closed (default status upon creation)
+ * @GDA_CONNECTION_STATUS_OPENING: the connection is currently being opened
+ * @GDA_CONNECTION_STATUS_IDLE: the connection is opened but not currently used
+ * @GDA_CONNECTION_STATUS_BUSY: the connection is opened and currently being used
+ *
+ * Indicates the current status of a connection. The possible status and the transitions between those status
+ * are indicated in the diagram below:
+ *  <mediaobject>
+ *    <imageobject role="html">
+ *      <imagedata fileref="connection-status.png" format="PNG" contentwidth="50mm"/>
+ *    </imageobject>
+ *    <textobject>
+ *      <phrase>GdaConnection's status and transitions between different status</phrase>
+ *    </textobject>
+ *  </mediaobject>
+ */
+typedef enum {
+	GDA_CONNECTION_STATUS_CLOSED,
+	GDA_CONNECTION_STATUS_OPENING,
+	GDA_CONNECTION_STATUS_IDLE,
+	GDA_CONNECTION_STATUS_BUSY
+} GdaConnectionStatus;
 
 struct _GdaConnectionClass {
 	GObjectClass          object_class;
 
 	/* signals */
+	void   (*status_changed)            (GdaConnection *obj, GdaConnectionStatus status);
 	void   (*error)                     (GdaConnection *cnc, GdaConnectionEvent *error);
-        void   (*conn_opened)               (GdaConnection *obj);
-        void   (*conn_to_close)             (GdaConnection *obj);
-        void   (*conn_closed)               (GdaConnection *obj);
+        void   (*opened)                    (GdaConnection *obj);
+        void   (*closed)                    (GdaConnection *obj);
 	void   (*dsn_changed)               (GdaConnection *obj);
 	void   (*transaction_status_changed)(GdaConnection *obj);
 
+
 	/*< private >*/
 	/* Padding for future expansion */
 	void (*_gda_reserved1) (void);
@@ -140,12 +149,7 @@ struct _GdaConnectionClass {
  * @GDA_CONNECTION_OPTIONS_READ_ONLY: this flag specifies that the connection to open should be in a read-only mode
  *                                    (this policy is not correctly enforced at the moment)
  * @GDA_CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE: this flag specifies that SQL identifiers submitted as input
- *                                    to Libgda have to keep their case sensitivity. 
- * @GDA_CONNECTION_OPTIONS_THREAD_SAFE: this flag specifies that the connection to open will be used 
- *                                     by several threads at once so it has to be thread safe
- * @GDA_CONNECTION_OPTIONS_THREAD_ISOLATED: this flag specifies that the connection to open will be used 
- *                                     by several threads at once and requests that the real connection be used
- *                                     only in a sub thread created specifically for it
+                                     to Libgda have to keep their case sensitivity. 
  * @GDA_CONNECTION_OPTIONS_AUTO_META_DATA: this flags specifies that if a #GdaMetaStore has been associated
  *                                     to the connection, then it is kept up to date with the evolutions in the
  *                                     database's structure. Be aware however that there are some drawbacks
@@ -166,22 +170,6 @@ struct _GdaConnectionClass {
  *       has to be conform to the database it will be used with</para></listitem>
  * </itemizedlist>
  *
- * Additional information about the GDA_CONNECTION_OPTIONS_THREAD_SAFE and GDA_CONNECTION_OPTIONS_THREAD_ISOLATED flags:
- * The GDA_CONNECTION_OPTIONS_THREAD_SAFE flag specifies that it has to be able to use the returned connection object from
- * several threads at once (locking is ensured by the #GdaConnection itself). Depending on the database provider's
- * implementation and on the native libraries it uses, the "normal" connection object might not respect this requirement,
- * and in this case a specific thread is started and used as the unique thread which will manipulate the actual connection,
- * while a "wrapper connection" is actually returned and used by the caller (that wrapper connection passes method calls
- * from the calling thread to the actual connection's specific thread, and gets the results back).
- *
- * The GDA_CONNECTION_OPTIONS_THREAD_ISOLATED forces using a specific thread and a "wrapper connection" even if the
- * "normal" connection would itself be thread safe; this is useful for example to be sure the asynchronous API can
- * always be used (see gda_connection_async_statement_execute()).
- *
- * Having a specific thread and a "wrapper connection" definitely has an impact on the performances (because it involves
- * messages passing between threads for every method call), so using the
- * GDA_CONNECTION_OPTIONS_THREAD_SAFE or GDA_CONNECTION_OPTIONS_THREAD_ISOLATED flags should be carefully considered.
- *
  * Note about the @GDA_CONNECTION_OPTIONS_AUTO_META_DATA flag:
  * <itemizedlist>
  *  <listitem><para>Every time a DDL statement is successfully executed, the associated meta data, if
@@ -194,8 +182,6 @@ typedef enum {
 	GDA_CONNECTION_OPTIONS_NONE = 0,
 	GDA_CONNECTION_OPTIONS_READ_ONLY = 1 << 0,
 	GDA_CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE = 1 << 1,
-	GDA_CONNECTION_OPTIONS_THREAD_SAFE = 1 << 2,
-	GDA_CONNECTION_OPTIONS_THREAD_ISOLATED = 1 << 3,
 	GDA_CONNECTION_OPTIONS_AUTO_META_DATA = 1 << 4
 } GdaConnectionOptions;
 
@@ -217,9 +203,11 @@ typedef enum {
  * @GDA_CONNECTION_FEATURE_UPDATABLE_CURSOR: test for updatable cursors support
  * @GDA_CONNECTION_FEATURE_USERS: test for users support
  * @GDA_CONNECTION_FEATURE_VIEWS: test for views support
+ * @GDA_CONNECTION_FEATURE_TRANSACTION_ISOLATION_READ_COMMITTED: test for READ COMMITTED transaction isolation level
+ * @GDA_CONNECTION_FEATURE_TRANSACTION_ISOLATION_READ_UNCOMMITTED: test for READ UNCOMMITTED transaction isolation level
+ * @GDA_CONNECTION_FEATURE_TRANSACTION_ISOLATION_REPEATABLE_READ: test for REPEATABLE READ transaction isolation level
+ * @GDA_CONNECTION_FEATURE_TRANSACTION_ISOLATION_SERIALIZABLE: test for SERIALIZABLE transaction isolation level
  * @GDA_CONNECTION_FEATURE_XA_TRANSACTIONS: test for distributed transactions support
- * @GDA_CONNECTION_FEATURE_MULTI_THREADING: test for native multi-threading support
- * @GDA_CONNECTION_FEATURE_ASYNC_EXEC: test if connection supports asynchronous execution
  * @GDA_CONNECTION_FEATURE_LAST: not used
  *
  * Used in gda_connection_supports_feature() and gda_server_provider_supports_feature() to test if a connection
@@ -241,11 +229,12 @@ typedef enum {
 	GDA_CONNECTION_FEATURE_UPDATABLE_CURSOR,
 	GDA_CONNECTION_FEATURE_USERS,
 	GDA_CONNECTION_FEATURE_VIEWS,
+	GDA_CONNECTION_FEATURE_TRANSACTION_ISOLATION_READ_COMMITTED,
+	GDA_CONNECTION_FEATURE_TRANSACTION_ISOLATION_READ_UNCOMMITTED,
+	GDA_CONNECTION_FEATURE_TRANSACTION_ISOLATION_REPEATABLE_READ,
+	GDA_CONNECTION_FEATURE_TRANSACTION_ISOLATION_SERIALIZABLE,
 	GDA_CONNECTION_FEATURE_XA_TRANSACTIONS,
 	
-	GDA_CONNECTION_FEATURE_MULTI_THREADING,
-	GDA_CONNECTION_FEATURE_ASYNC_EXEC,
-
 	GDA_CONNECTION_FEATURE_LAST
 } GdaConnectionFeature;
 
@@ -271,31 +260,60 @@ typedef enum {
 } GdaConnectionMetaType;
 
 
-GType                gda_connection_get_type             (void) G_GNUC_CONST;
-GdaConnection       *gda_connection_open_from_dsn        (const gchar *dsn, const gchar *auth_string,
-							  GdaConnectionOptions options, GError **error);
-GdaConnection       *gda_connection_open_from_string     (const gchar *provider_name, 
+GdaConnection       *gda_connection_open_from_dsn_name   (const gchar *dsn_name,
+                                                          const gchar *auth_string,
+                                                          GdaConnectionOptions options,
+                                                          GError **error);
+
+GdaConnection       *gda_connection_open_from_dsn        (GdaDsnInfo *dsn,
+                                                          const gchar *auth_string,
+                                                          GdaConnectionOptions options,
+                                                          GError **error);
+GdaConnection       *gda_connection_open_from_string     (const gchar *provider_name,
 							  const gchar *cnc_string, const gchar *auth_string,
 							  GdaConnectionOptions options, GError **error);
-GdaConnection       *gda_connection_new_from_dsn         (const gchar *dsn, const gchar *auth_string,
-							  GdaConnectionOptions options, GError **error);
+GdaConnection       *gda_connection_new_from_dsn_name     (const gchar *dsn_name,
+                                                           const gchar *auth_string,
+                                                           GdaConnectionOptions options,
+                                                           GError **error);
+
+GdaConnection       *gda_connection_new_from_dsn          (GdaDsnInfo *dsn,
+                                                           const gchar *auth_string,
+                                                           GdaConnectionOptions options,
+                                                           GError **error);
+
 GdaConnection       *gda_connection_new_from_string      (const gchar *provider_name, 
 							  const gchar *cnc_string, const gchar *auth_string,
 							  GdaConnectionOptions options, GError **error);
+
 gboolean             gda_connection_open                 (GdaConnection *cnc, GError **error);
-void                 gda_connection_close                (GdaConnection *cnc);
-void                 gda_connection_close_no_warning     (GdaConnection *cnc);
+typedef void (*GdaConnectionOpenFunc) (GdaConnection *cnc, guint job_id, gboolean result, GError *error, gpointer data);
+guint                gda_connection_open_async           (GdaConnection *cnc, GdaConnectionOpenFunc callback, gpointer data, GError **error);
+
+gboolean             gda_connection_close                (GdaConnection *cnc, GError **error);
 gboolean             gda_connection_is_opened            (GdaConnection *cnc);
+GdaConnectionStatus  gda_connection_get_status           (GdaConnection *cnc);
 
 GdaConnectionOptions gda_connection_get_options          (GdaConnection *cnc);
 
+void                 gda_connection_set_main_context     (GdaConnection *cnc, GThread *thread, GMainContext *context);
+GMainContext        *gda_connection_get_main_context     (GdaConnection *cnc, GThread *thread);
+
 GdaServerProvider   *gda_connection_get_provider         (GdaConnection *cnc);
 const gchar         *gda_connection_get_provider_name    (GdaConnection *cnc);
 
 GdaServerOperation  *gda_connection_create_operation     (GdaConnection *cnc, GdaServerOperationType type,
                                                           GdaSet *options, GError **error);
+
 gboolean             gda_connection_perform_operation    (GdaConnection *cnc, GdaServerOperation *op, GError **error);
                                                           
+GdaServerOperation  *gda_connection_prepare_operation_create_table_v          (GdaConnection *cnc, const gchar *table_name, GError **error, ...);
+GdaServerOperation  *gda_connection_prepare_operation_create_table        (GdaConnection *cnc, const gchar *table_name, GList *arguments, GError **error);
+GdaServerOperation  *gda_connection_prepare_operation_drop_table            (GdaConnection *cnc, const gchar *table_name, GError **error);
+gchar               *gda_connection_operation_get_sql_identifier_at (GdaConnection *cnc,
+              GdaServerOperation *op, const gchar *path_format, GError **error, ...);
+gchar               *gda_connection_operation_get_sql_identifier_at_path (GdaConnection *cnc, GdaServerOperation *op,
+						 const gchar *path, GError **error);
 const gchar         *gda_connection_get_dsn              (GdaConnection *cnc);
 const gchar         *gda_connection_get_cnc_string       (GdaConnection *cnc);
 const gchar         *gda_connection_get_authentication   (GdaConnection *cnc);
@@ -304,7 +322,7 @@ gboolean             gda_connection_get_date_format      (GdaConnection *cnc, GD
 							  GError **error);
 
 GdaStatement        *gda_connection_parse_sql_string     (GdaConnection *cnc, const gchar *sql, GdaSet **params,
-                    GError **error);
+							  GError **error);
 
 /*
  * Quick commands execution
@@ -317,20 +335,20 @@ gint                gda_connection_execute_non_select_command    (GdaConnection
  */
 gboolean            gda_connection_insert_row_into_table        (GdaConnection *cnc, const gchar *table, GError **error, ...);
 gboolean            gda_connection_insert_row_into_table_v      (GdaConnection *cnc, const gchar *table,
-						      GSList *col_names, GSList *values,
-						      GError **error);
+								 GSList *col_names, GSList *values,
+								 GError **error);
 
 gboolean            gda_connection_update_row_in_table          (GdaConnection *cnc, const gchar *table,
-						      const gchar *condition_column_name,
-						      GValue *condition_value, GError **error, ...);
+								 const gchar *condition_column_name,
+								 GValue *condition_value, GError **error, ...);
 gboolean            gda_connection_update_row_in_table_v        (GdaConnection *cnc, const gchar *table,
-						      const gchar *condition_column_name,
-						      GValue *condition_value,
-						      GSList *col_names, GSList *values,
-						      GError **error);
+								 const gchar *condition_column_name,
+								 GValue *condition_value,
+								 GSList *col_names, GSList *values,
+								 GError **error);
 gboolean            gda_connection_delete_row_from_table        (GdaConnection *cnc, const gchar *table,
-						      const gchar *condition_column_name,
-						      GValue *condition_value, GError **error);
+								 const gchar *condition_column_name,
+								 GValue *condition_value, GError **error);
 
 const GList         *gda_connection_get_events           (GdaConnection *cnc);
 
@@ -360,15 +378,6 @@ GdaDataModel        *gda_connection_statement_execute_select_full (GdaConnection
 gint                 gda_connection_statement_execute_non_select (GdaConnection *cnc, GdaStatement *stmt,
 								  GdaSet *params, GdaSet **last_insert_row, GError **error);
 
-/* asynchronous execution */
-guint                gda_connection_async_statement_execute (GdaConnection *cnc, GdaStatement *stmt, GdaSet *params, 
-							     GdaStatementModelUsage model_usage, GType *col_types,
-							     gboolean need_last_insert_row, 
-							     GError **error);
-GObject             *gda_connection_async_fetch_result      (GdaConnection *cnc, guint task_id, GdaSet **last_insert_row, GError **error);
-gboolean             gda_connection_async_cancel            (GdaConnection *cnc, guint task_id, GError **error);
-
-
 /* repetitive statement */
 GSList             *gda_connection_repetitive_statement_execute (GdaConnection *cnc, GdaRepetitiveStatement *rstmt,
 								 GdaStatementModelUsage model_usage, GType *col_types,
@@ -395,6 +404,7 @@ GdaDataModel        *gda_connection_get_meta_store_data  (GdaConnection *cnc, Gd
 							  GError **error, gint nb_filters, ...);
 GdaDataModel        *gda_connection_get_meta_store_data_v(GdaConnection *cnc, GdaConnectionMetaType meta_type,
 							  GList* filters, GError **error);
+GdaDbCatalog        *gda_connection_create_db_catalog    (GdaConnection *cnc);
 G_END_DECLS
 
 #endif
