@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2008 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2008 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2018 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,35 +23,13 @@
 
 #include <glib-object.h>
 #include <libgda/gda-data-model.h>
-#include <libgda/gda-meta-store.h>
 #include <libgda/gda-decl.h>
 #include <libgda/gda-attributes-manager.h>
 
 G_BEGIN_DECLS
 
 #define GDA_TYPE_META_STRUCT          (gda_meta_struct_get_type())
-#define GDA_META_STRUCT(obj)          G_TYPE_CHECK_INSTANCE_CAST (obj, gda_meta_struct_get_type(), GdaMetaStruct)
-#define GDA_META_STRUCT_CLASS(klass)  G_TYPE_CHECK_CLASS_CAST (klass, gda_meta_struct_get_type (), GdaMetaStructClass)
-#define GDA_IS_META_STRUCT(obj)       G_TYPE_CHECK_INSTANCE_TYPE (obj, gda_meta_struct_get_type ())
-
-/* error reporting */
-extern GQuark gda_meta_struct_error_quark (void);
-#define GDA_META_STRUCT_ERROR gda_meta_struct_error_quark ()
-
-typedef enum {
-	GDA_META_STRUCT_UNKNOWN_OBJECT_ERROR,
-        GDA_META_STRUCT_DUPLICATE_OBJECT_ERROR,
-        GDA_META_STRUCT_INCOHERENCE_ERROR,
-	GDA_META_STRUCT_XML_ERROR
-} GdaMetaStructError;
-
-
-/* struct for the object's data */
-struct _GdaMetaStruct
-{
-	GObject                object;
-	GdaMetaStructPrivate  *priv;
-};
+G_DECLARE_DERIVABLE_TYPE(GdaMetaStruct, gda_meta_struct, GDA, META_STRUCT, GObject)
 
 /* struct for the object's class */
 struct _GdaMetaStructClass
@@ -65,6 +44,17 @@ struct _GdaMetaStructClass
 	void (*_gda_reserved4) (void);
 };
 
+/* error reporting */
+extern GQuark gda_meta_struct_error_quark (void);
+#define GDA_META_STRUCT_ERROR gda_meta_struct_error_quark ()
+
+typedef enum {
+	GDA_META_STRUCT_UNKNOWN_OBJECT_ERROR,
+        GDA_META_STRUCT_DUPLICATE_OBJECT_ERROR,
+        GDA_META_STRUCT_INCOHERENCE_ERROR,
+	GDA_META_STRUCT_XML_ERROR
+} GdaMetaStructError;
+
 /**
  * GdaMetaDbObjectType:
  * @GDA_META_DB_UNKNOWN: unknown type
@@ -256,7 +246,7 @@ typedef struct {
  * @gtype: the detected column's #GType
  * @pkey: tells if the column is part of a primary key
  * @nullok: tells if the column can be %NULL
- * @default_value: (allow-none): the column's default value, represented as a valid SQL value (surrounded by simple quotes for strings, ...), or %NULL if column has no default value
+ * @default_value: (nullable): the column's default value, represented as a valid SQL value (surrounded by simple quotes for strings, ...), or %NULL if column has no default value
  *
  * This structure represents a table of view's column, its contents must not be modified.
  */
@@ -268,6 +258,8 @@ typedef struct {
 	gboolean      pkey;
 	gboolean      nullok;
 	gchar        *default_value;
+	gboolean      auto_incement;
+	gchar        *desc;
 
 	/*< private >*/
 	/* Padding for future expansion */
@@ -287,21 +279,6 @@ typedef struct {
  */
 #define GDA_META_TABLE_COLUMN(col) ((GdaMetaTableColumn*)(col))
 
-const GValue *gda_meta_table_column_get_attribute (GdaMetaTableColumn *tcol, const gchar *attribute);
-void          gda_meta_table_column_set_attribute (GdaMetaTableColumn *tcol, const gchar *attribute, const GValue *value,
-						   GDestroyNotify destroy);
-/**
- * gda_meta_table_column_set_attribute_static:
- * @column: a #GdaMetaTableColumn
- * @attribute: attribute's name
- * @value: (allow-none): a #GValue, or %NULL
- *
- * This function is similar to gda_meta_table_column_set_attribute() but for static strings
- */
-#define gda_meta_table_column_set_attribute_static(column,attribute,value) gda_meta_table_column_set_attribute((column),(attribute),(value),NULL)
-
-void          gda_meta_table_column_foreach_attribute (GdaMetaTableColumn *tcol, GdaAttributesManagerFunc func, gpointer data);
-
 /**
  * GdaMetaForeignKeyPolicy:
  * @GDA_META_FOREIGN_KEY_UNKNOWN: unspecified policy
@@ -490,8 +467,9 @@ typedef struct {
  *  </programlisting>
  */
 
-GType               gda_meta_struct_get_type           (void) G_GNUC_CONST;
-GdaMetaStruct      *gda_meta_struct_new                (GdaMetaStore *store, GdaMetaStructFeature features);
+gboolean            gda_meta_struct_load_from_xml_file (GdaMetaStruct *mstruct, const gchar *catalog,
+							const gchar *schema, 
+							const gchar *xml_spec_file, GError **error);
 GdaMetaDbObject    *gda_meta_struct_complement         (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
 							const GValue *catalog, const GValue *schema, const GValue *name,
 							GError **error);
